package net.wanji.opt.service.impl;


import com.alibaba.fastjson.JSONObject;
import io.swagger.models.auth.In;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.enums.TurnConvertEnum;
import net.wanji.common.framework.Constants;
import net.wanji.common.framework.dubbointerface.impl.BaseDubboInterfaceImpl;
import net.wanji.common.framework.exception.DubboProviderException;
import net.wanji.common.framework.mapper.BaseInterfaceMapper;
import net.wanji.common.gts.Tools;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.databus.dao.mapper.BaseCrossInfoMapper;
import net.wanji.databus.po.BaseCrossInfoPO;
import net.wanji.databus.po.CrossDataHistPO;
import net.wanji.databus.po.CrossDirDataHistPO;
import net.wanji.databus.po.CrossTurnDataHistPO;
import net.wanji.databus.po.CrossTurnInfoPO;
import net.wanji.databus.po.TBaseCrossInfo;
import net.wanji.opt.bo.BottomMenuBO;
import net.wanji.opt.common.ArithOfBigDecmial;
import net.wanji.opt.common.EsDateIndexUtil;
import net.wanji.opt.common.enums.TimeGranularityEnum;
import net.wanji.opt.constant.ServiceLevelEnum;
import net.wanji.opt.dao.mapper.GreenwaveHistoryMapper;
import net.wanji.opt.entity.GreenwaveHist;
import net.wanji.opt.po.base.CrossLaneDataHistPoExtend;
import net.wanji.opt.po.trend.GreenwaveCrossTrendPo;
import net.wanji.opt.service.GreenwaveHistProvider;
import net.wanji.opt.service.MainlineEvaluateService;
import net.wanji.opt.vo.GreenWaveRunStateVO;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.DubboService;
import org.joda.time.DateTime;
import org.joda.time.Seconds;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Random;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;


/**
 * <p>
 * 绿波历史数据
 * </p>
 *
 * @Author fengyi
 * @Date 2024-11-18
 */
@Slf4j
@Component
@DubboService
public class GreenwaveHistProviderImpl extends BaseDubboInterfaceImpl<GreenwaveHist> implements GreenwaveHistProvider {

    @Resource
    private GreenwaveHistoryMapper greenwaveHistoryMapper;
    @Resource
    private MainlineEvaluateService mainlineEvaluateService;

    @Resource
    BaseCrossInfoMapper baseCrossInfoMapper;

    @Value("${data.mock.flag}")
    private boolean mockFlag;

    @Override
    public BaseInterfaceMapper<GreenwaveHist> getBaseInterfaceMapper() {
        return this.greenwaveHistoryMapper;
    }


    @Override
    public JSONObject findStatisticIndex(String greenId, String startTime, String endTime, String type) {
        Map<String, Object> params = new HashMap<>();
        params.put("greenId", greenId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);

        List<GreenwaveHist> list = greenwaveHistoryMapper.findStatisticIndex(params);
        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<String, List<GreenwaveHist>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getRoadDirection(), TreeMap::new, Collectors.toList()));

        List<Map<String, Object>> turnList = new ArrayList<>();

        for (Map.Entry<String, List<GreenwaveHist>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();

            //按方向排序
            List<GreenwaveHist> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("direction", key);
            mapList.put("list", value);
            allList.add(mapList);

            for (GreenwaveHist po : value) {
                Map<String, Object> turnMap = new HashMap<>();
                turnMap.put("direction", po.getDirection());
                if (!turnList.contains(turnMap)) {
                    turnList.add(turnMap);
                }
                //提取时间
                sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
            }
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("type", type);
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("directionList", new int[]{0, 1, 2});

        return jsonObject;

    }

    @Override
    public List<Map<String, Object>> findGreenCross(Integer greenId) throws DubboProviderException {
        Map<String, Object> params = new HashMap<>();
        params.put("greenId", greenId);

        List<Map<String, Object>> list = greenwaveHistoryMapper.findGreenCross(params);
        String dirs = list.stream().map(o -> o.get("inDir").toString()).distinct().collect(Collectors.joining(","));


        return list;
    }

    @Override
    public JSONObject findUnCoordinateCrossIndex(String crossId, String greenDir, String startTime, String endTime) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("greenDir", greenDir);

        List<Map<String, Object>> getList = greenwaveHistoryMapper.findUnCoordinateCrossIndex(params);

        List<GreenwaveCrossTrendPo> list = new ArrayList<>();
        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        for (Map<String, Object> map : getList) {
            GreenwaveCrossTrendPo po = new GreenwaveCrossTrendPo();
            String crossNo = map.get("cross_id").toString();
            String st = map.get("start_time").toString();
            po.setCrossId(crossNo);
            po.setStartTime(DateTime.parse(st, DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")).toDate());
            //提取时间
            sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));

            String indexList = map.get("index_list").toString();

            List<GreenwaveCrossTrendPo> finalList = new ArrayList<>();
            Arrays.stream(indexList.split("@")).forEach(o -> {
                String[] sps = o.split(",");
                String type = sps[0];
                String queueLength = sps[1];
                String stopTimes = sps[2];
                String flow = sps[3];
                if (Objects.equals("1", type)) {
                    //非协调方向
                    po.setFlow(new BigDecimal(flow));
                    po.setQueueLength(new BigDecimal(queueLength));
                    po.setStopTimes(new BigDecimal(stopTimes));
                } else if (Objects.equals("2", type)) {
                    //协调方向
                    po.setCordFlow(new BigDecimal(flow));
                    po.setCordQueueLength(new BigDecimal(queueLength));
                    po.setCordStopTimes(new BigDecimal(stopTimes));
                }
                finalList.add(po);
            });

            list.addAll(finalList);

        }
        //排序
        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();
        Map<String, Object> mapList = new HashMap<>();
        //mapList.put("direction", key);
        mapList.put("list", list);
        allList.add(mapList);

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);

        return jsonObject;

    }

    @Override
    public JSONObject findGreenWaveCrossDirIndex(String crossId, String startTime, String endTime, String groupType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        List<CrossDirDataHistPO> list = greenwaveHistoryMapper.findGreenWaveCrossDirIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<Integer, List<CrossDirDataHistPO>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getDirType(), TreeMap::new, Collectors.toList()));

        List<Map<String, Object>> turnList = new ArrayList<>();

        for (Map.Entry<Integer, List<CrossDirDataHistPO>> entry : groupByDir.entrySet()) {
            Integer key = entry.getKey();

            //按方向排序
            List<CrossDirDataHistPO> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("direction", key);
            mapList.put("list", value);
            allList.add(mapList);

            for (CrossDirDataHistPO po : value) {
                Map<String, Object> turnMap = new HashMap<>();
                turnMap.put("direction", po.getDirType());
                if (!turnList.contains(turnMap)) {
                    turnList.add(turnMap);
                }
                //提取时间
                sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
            }
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("dirList", groupByDir.keySet());

        return jsonObject;
    }


    @Override
    public JSONObject findGreenWaveCrossIndex(String crossId, String startTime, String endTime, String groupType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        List<CrossDataHistPO> list = greenwaveHistoryMapper.findGreenWaveCrossIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = new TreeSet<>();

        for (CrossDataHistPO po : list) {
            //提取时间
            sortedSet.add(DateUtil.formatDate(po.getStartTime(), "HH:mm"));
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", list);

        return jsonObject;
    }

    @Override
    public List<GreenWaveRunStateVO> findGreenWaveRunState(Integer greenId, String crossId, String startTime, String endTime, String eventType) {
        Map<String, Object> params = new HashMap<>();
        params.put("greenId", greenId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("crossId", crossId);

        if (Objects.nonNull(eventType)) {
            String[] arr = StringUtils.split(eventType, ",");
            params.put("eventTypeList", Arrays.asList(arr));
        }

        int[] stateList = new int[]{705, 706};

        List<GreenWaveRunStateVO> list = greenwaveHistoryMapper.findGreenWaveRunState(params);
        Map<Integer, GreenWaveRunStateVO> groupList = list.stream().collect(Collectors.toMap(o -> o.getState(), o -> o));


        DateTime startDateTime = DateTime.parse(startTime, DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));
        DateTime endDateTime = DateTime.parse(endTime, DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss"));

        for (int state : stateList) {
            if (!groupList.containsKey(state)) {
                GreenWaveRunStateVO tmp = new GreenWaveRunStateVO();
                tmp.setState(state);
                tmp.setGreenId(greenId);
                list.add(tmp);
            }
        }
        int totalTime = Seconds.secondsBetween(startDateTime, endDateTime).getSeconds();
        for (GreenWaveRunStateVO vo : list) {
//            if (vo.getDuration()<=0) {
//                vo.setDuration(getRandomValue(totalTime / 10, 120));
//            }
            vo.setTotalTime(totalTime);
        }

        return list;
    }

    @Override
    public JSONObject findGreenWaveCrossTurnIndex(String crossId, String startTime, String endTime, String groupType) throws DubboProviderException {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        List<CrossTurnDataHistPO> list = greenwaveHistoryMapper.findGreenWaveCrossTurnIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis(groupType, startTime, endTime);
        //======================================================================================================//
        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();
        //按时间分组分组
        Map<String, List<CrossTurnDataHistPO>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getInDir() + "_" + o.getTurnType(), TreeMap::new, Collectors.toList()));


        for (Map.Entry<String, List<CrossTurnDataHistPO>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();
            String[] sps = key.split("_");
            Integer dirType = Integer.parseInt(sps[0]);
            String dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
            String turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[1]));
            String dirTurnDesc = dirName + "进口" + turnDesc;

            //按方向排序
            List<CrossTurnDataHistPO> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
            //有实际数据的时刻点
            Set<String> timeList = value.stream().map(po -> DateUtil.formatDate(po.getStartTime(), EsDateIndexUtil.YMD_HM_FORMATTER)).collect(Collectors.toSet());
            //补充缺少时段数据，保留时段字段默认值
            for (String timeSec : sortedSet) {
                if (!timeList.contains(timeSec)) {
                    CrossTurnDataHistPO tmp = new CrossTurnDataHistPO();
                    tmp.setStartTime(DateTime.parse(timeSec, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_HM_FORMATTER)).toDate());
                    value.add(tmp);
                }
            }
            value = value.stream().sorted(Comparator.comparing(CrossTurnDataHistPO::getStartTime)).collect(Collectors.toList());

//            sortedSet.clear();
//            value.forEach(o->sortedSet.add(DateUtil.formatDate(o.getStartTime(),"HH:mm")));

            Map<String, Object> mapList = new HashMap<>();
            mapList.put("dirTurnCode", key);
            mapList.put("dirTurnDesc", dirTurnDesc);
            mapList.put("list", value);
            allList.add(mapList);

        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet.stream().map(o -> o.substring(o.indexOf(" ") + 1, o.lastIndexOf(":"))));
        jsonObject.put("dataList", allList);
        jsonObject.put("dirList", groupByDir.keySet());


        return jsonObject;
    }

    @Override
    public JSONObject findCrossObjectIndex(String crossId, String startTime, String endTime, String groupType, Integer objectType, String directionName) throws DubboProviderException {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);
        params.put("objectType", objectType);
        if (StringUtils.isNotBlank(directionName)) {
            params.put("dir", BaseEnum.SignalDirectionEnum.getCodeByName(directionName.substring(0, directionName.indexOf("进口"))));
        }
        //存放时段
        List<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxisAll(groupType, startTime, endTime);
        //======================================================================================================//
        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();
        long st1 = System.currentTimeMillis();
        List<CrossLaneDataHistPoExtend> list = greenwaveHistoryMapper.findCrossObjectIndex(params);
        long et1 = System.currentTimeMillis();
        log.info("findCrossObjectIndex查耗时：{}ms，params:{}",et1-st1,params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
        //按时间分组分组
        Map<String, List<CrossLaneDataHistPoExtend>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getCrossId() + "_" + o.getLaneNo() + "_" + o.getDirType() + "_" + o.getTurnType(), TreeMap::new, Collectors.toList()));

        Set<String> hasDataScopeList = new HashSet<>();

        for (Map.Entry<String, List<CrossLaneDataHistPoExtend>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();
            String[] sps = key.split("_");
            String laneNo = sps[1];
            Integer dirType = null;
            String dirName = null;
            String turnDesc = null;
            String dirTurnDesc = null;

            Map<String, Object> mapList = new HashMap<>();
            if (Objects.equals(4, objectType)) {
                //路口级
                TBaseCrossInfo tBaseCrossInfo = baseCrossInfoMapper.selectByPrimaryKey(sps[0]);
                dirTurnDesc = tBaseCrossInfo.getName();
                dirType = 0;
            } else if (Objects.equals(3, objectType)) {
                //车道级
                mapList.put("laneNo", laneNo);
                dirType = Integer.parseInt(sps[2]);
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[3]));
                dirTurnDesc = dirName + "进口" + turnDesc + laneNo + "车道";
            } else if (Objects.equals(2, objectType)) {
                //转向级
                dirType = Integer.parseInt(sps[2]);
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[3]));
                dirTurnDesc = dirName + "进口" + turnDesc;
            } else if (Objects.equals(1, objectType)) {
                //方向级
                dirType = Integer.parseInt(sps[2]);
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                dirTurnDesc = dirName + "进口";
            }

            hasDataScopeList.add(dirTurnDesc);

            //补充缺少时段数据，保留时段字段默认值
            List<CrossLaneDataHistPoExtend> value = this.processData(entry, groupType, sortedSet, startTime);


            if (Objects.equals(2, objectType)) {
                value = value.stream().sorted(Comparator.comparing(o -> o.getTurnType())).collect(Collectors.toList());
            } else if (Objects.equals(3, objectType)) {
                value = value.stream().sorted(Comparator.comparing(o -> o.getLaneNo())).collect(Collectors.toList());
            }
            if (Objects.equals(3, objectType)){
                //车道指标mock数据
                this.mockData(value);
            }

            mapList.put("scopeCode", key);
            mapList.put("dirType", dirType);
            mapList.put("scopeName", dirTurnDesc);
            mapList.put("list", value);
            allList.add(mapList);

        }
//        BottomMenuBO bo = new BottomMenuBO();
//        bo.setCrossId(crossId);
//        bo.setScope(objectType);
//        List<String> scopeList =  mainlineEvaluateService.bottomMenu(bo);

        List<String> scopeList = hasDataScopeList.stream().sorted().collect(Collectors.toList());

        if (Objects.nonNull(scopeList)) {
            //补充一条时段都没有的
            for (String scopeName1 : scopeList) {
                if (!hasDataScopeList.contains(scopeName1)) {
                    Map<String, Object> mapList = new HashMap<>();
                    mapList.put("scopeCode", "");
                    mapList.put("scopeName", scopeName1);
                    mapList.put("dirType", BaseEnum.SignalDirectionEnum.getCodeByName(scopeName1.substring(0, scopeName1.indexOf("进口"))));
                    List<CrossLaneDataHistPoExtend> tmpList = new ArrayList<>();
                    //补充所有时段字段对象
                    for (String time : sortedSet) {
                        CrossLaneDataHistPoExtend po = new CrossLaneDataHistPoExtend();
                        try {
                            po.setStartTime(DateUtil.parse(time, "yyyy-MM-dd HH:mm:ss"));
                        } catch (ParseException e) {
                            e.printStackTrace();
                        }
                        po.setTimeAxis(time);
                        tmpList.add(po);
                    }
                    if (Objects.equals(3, objectType)){
                        //车道指标mock数据
                        this.mockData(tmpList);
                    }
                    mapList.put("list", tmpList);
                    allList.add(mapList);
                }
            }
        }
        allList = allList.stream().sorted(Comparator.comparing(o -> Integer.valueOf(o.get("dirType").toString()))).collect(Collectors.toList());
        if (params.get("dir") != null) {
            allList = allList.stream().filter(o -> Objects.equals(params.get("dir").toString(), o.get("dirType").toString())).collect(Collectors.toList());
        }
        List<String> timeList = this.getTimeAxisList(groupType,sortedSet);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", timeList);
        jsonObject.put("dateTimeList", sortedSet);

        jsonObject.put("dataList", allList);
        jsonObject.put("scopeList", scopeList);

        log.info("findCrossObjectIndex总查耗时：{}ms，params:{}",System.currentTimeMillis()-st1,params);

        return jsonObject;
    }

    private void mockData(List<CrossLaneDataHistPoExtend> list) {
        if (mockFlag) {
            int maxFlow = list.stream().filter(o -> Objects.nonNull(o.getFlow())).mapToInt(CrossLaneDataHistPoExtend::getFlow).max().orElse(0);

            for (CrossLaneDataHistPoExtend po : list) {
                if (po.getFlow() == null) {
                    po.setFlow(0);
                }
                double r = 0;
                if (maxFlow == 0) {
                    r = 0;
                    po.setDelayTime(0);
                    po.setSpeed(0D);
                } else {
                    r = ArithOfBigDecmial.div(po.getFlow(), maxFlow, 2);
                    po.setDelayTime((int)ArithOfBigDecmial.round(5 + r * 20, 0));
                    po.setSpeed(ArithOfBigDecmial.round(30 + 30 * (1 - r), 2));
                }
                po.setSturation(ArithOfBigDecmial.round(r * 100, 2));
                po.setGreenLightEfficiency(ArithOfBigDecmial.round(r * 0.8 * 100, 2));
                po.setVehicleLengthRatioMean(ArithOfBigDecmial.round(r * 0.9 * 100, 2));
                if (po.getFlow()==0){
                    po.setDelayTime(0);
                }
//                DateTime dateTime = new DateTime(po.getStartTime());
//                int hour = dateTime.hourOfDay().get();
//                if (hour < 6) {
//                    po.setSpeed(net.wanji.opt.common.Tools.getDoubleRandomValue(80, 50));
//                } else if (hour < 8) {
//                    po.setSpeed(net.wanji.opt.common.Tools.getDoubleRandomValue(50, 30));
//                } else if (hour < 17) {
//                    po.setSpeed(net.wanji.opt.common.Tools.getDoubleRandomValue(60, 40));
//                } else {
//                    po.setSpeed(net.wanji.opt.common.Tools.getDoubleRandomValue(50, 30));
//                }
            }
        }
    }

    @Override
    public JSONObject findCrossAvgIndex(String crossId, String startTime, String endTime, String groupType, Integer objectType) throws DubboProviderException {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);
        params.put("objectType", objectType);

        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis(groupType, startTime, endTime);
        //======================================================================================================//
        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();

        List<CrossLaneDataHistPoExtend> list = greenwaveHistoryMapper.findCrossObjectIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
        //路口方向分钟
        Map<String, List<CrossLaneDataHistPoExtend>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getCrossId() + "_" + o.getDirType(), TreeMap::new, Collectors.toList()));

        Set<String> hasDataScopeList = new HashSet<>();


        BottomMenuBO bo = new BottomMenuBO();
        bo.setCrossId(crossId);
        bo.setScope(objectType);
        List<String> scopeList = mainlineEvaluateService.bottomMenu(bo);

        TBaseCrossInfo tBaseCrossInfo = baseCrossInfoMapper.selectByPrimaryKey(crossId);
        String crossName = tBaseCrossInfo.getName();

        for (Map.Entry<String, List<CrossLaneDataHistPoExtend>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();
            String[] sps = key.split("_");
            String cid = sps[0];
            String dirType = sps[1];

            String dirName = null;
            if (!Objects.equals("null", dirType)) {
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(Integer.valueOf(dirType));
            }
            String dirTurnDesc = dirName + "进口";
            Map<String, Object> mapList = new HashMap<>();

            List<CrossLaneDataHistPoExtend> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getTurnType())).collect(Collectors.toList());
            //========================路口级服务水平加权平均 start=================================================================//
            String serviceLevelList = value.get(0).getServiceLevel();
            StringBuilder serviceLevelBuf = new StringBuilder();
            if (serviceLevelList != null) {
                Map<String, Long> serviceGroup = Arrays.stream(serviceLevelList.split(",")).collect(Collectors.groupingBy(String::valueOf, Collectors.counting()));
                double total = 0;
                for (Map.Entry<String, Long> entry1 : serviceGroup.entrySet()) {
                    String key1 = entry1.getKey();
                    total += ServiceLevelEnum.getByType(key1).getVal() * entry1.getValue();
                }
                //服务水平加权平均计算：给服务水平(等级得分*该等级的数量)累加/总数量
                int avgService = (int) Math.round(total / value.size());
                String avgServiceLevel = ServiceLevelEnum.getByVal(avgService) == null ? "A" : ServiceLevelEnum.getByVal(avgService).getType();
                serviceLevelBuf.append(avgServiceLevel);
            }
            //========================路口级服务水平加权平均 end=================================================================//
            value.forEach(o -> {
                o.setServiceLevel(serviceLevelBuf.toString());
                o.setDirTypeName(dirTurnDesc);
                if (Objects.equals(2, objectType)) {
                    o.setTurnTypeName(TurnConvertEnum.getDescByKey(o.getTurnType()));
                    hasDataScopeList.add(dirTurnDesc + o.getTurnTypeName());
                } else if (Objects.equals(3, objectType)) {
                    o.setTurnTypeName(TurnConvertEnum.getDescByKey(o.getTurnType()) + o.getLaneNo() + "车道");
                    hasDataScopeList.add(dirTurnDesc + o.getTurnTypeName());
                } else {
                    hasDataScopeList.add(dirTurnDesc);
                }
            });
            //===============补充某个方向有数据但是不完整的================================================================================//
            if (Objects.nonNull(scopeList)) {
                //补充
                for (String scopeName1 : scopeList) {
                    if (!hasDataScopeList.contains(scopeName1) && scopeName1.contains(dirTurnDesc)) {
                        CrossLaneDataHistPoExtend tmp = new CrossLaneDataHistPoExtend();
                        tmp.setCrossId(cid);
                        tmp.setDirType(BaseEnum.SignalDirectionEnum.getCodeByName(scopeName1.substring(0, scopeName1.indexOf("进口"))));
                        tmp.setDirTypeName(dirTurnDesc);
                        tmp.setTurnTypeName(scopeName1.substring(scopeName1.indexOf(dirTurnDesc) + dirTurnDesc.length()));
                        tmp.setTurnType(TurnConvertEnum.getKeyByDesc(tmp.getTurnTypeName()));
                        hasDataScopeList.add(tmp.getDirTypeName() + tmp.getTurnTypeName());
                        //利用正则表达式取车道号
                        if (Objects.equals(3, objectType)) {
                            Pattern pattern = Pattern.compile("\\d+");
                            Matcher matcher = pattern.matcher(scopeName1);
                            if (matcher.find()) {
                                Integer laneNo = Integer.valueOf(matcher.group());
                                tmp.setLaneNo(laneNo);
                                tmp.setTurnType(TurnConvertEnum.getKeyByDesc(tmp.getTurnTypeName().substring(0, tmp.getTurnTypeName().indexOf(laneNo.toString()))));
                                hasDataScopeList.add(tmp.getDirTypeName() + tmp.getTurnTypeName());
                            }
                        }
                        value.add(tmp);
                    }
                }
            }
            //===============补充某个方向有数据但是不完整的================================================================================//
            if (Objects.equals(2, objectType)) {
                value = value.stream().sorted(Comparator.comparing(o -> o.getTurnType())).collect(Collectors.toList());
            } else if (Objects.equals(3, objectType)) {
                value = value.stream().sorted(Comparator.comparing(o -> o.getLaneNo())).collect(Collectors.toList());
            }

            mapList.put("scopeCode", key);
            mapList.put("dirName", dirTurnDesc);
            mapList.put("scopeName", crossName);
            mapList.put("dirType", BaseEnum.SignalDirectionEnum.getCodeByName(dirTurnDesc.substring(0, dirTurnDesc.indexOf("进口"))));
            mapList.put("list", value);
            allList.add(mapList);

        }
        //====================补充方向上无任何车道数据====================================================================================//
        if (Objects.nonNull(scopeList)) {

            Map<Integer, List<CrossLaneDataHistPoExtend>> tmpListMap = new TreeMap<>();
            for (String scopeName1 : scopeList) {
                if (!hasDataScopeList.contains(scopeName1)) {
                    Pattern pattern = Pattern.compile("\\d+");
                    Matcher matcher = pattern.matcher(scopeName1);

                    String dirName = scopeName1.substring(0, scopeName1.indexOf("进口") + 2);
                    String turnDesc = scopeName1.substring(dirName.length());

                    CrossLaneDataHistPoExtend tmp = new CrossLaneDataHistPoExtend();
                    tmp.setCrossId(crossId);
                    tmp.setDirType(BaseEnum.SignalDirectionEnum.getCodeByName(scopeName1.substring(0, scopeName1.indexOf("进口"))));
                    tmp.setDirTypeName(dirName);
                    tmp.setTurnTypeName(turnDesc);

                    if (matcher.find()) {
                        Integer laneNo = Integer.valueOf(matcher.group());
                        tmp.setTurnType(TurnConvertEnum.getKeyByDesc(tmp.getTurnTypeName().substring(0, tmp.getTurnTypeName().indexOf(laneNo.toString()))));
                    } else {
                        tmp.setTurnType(TurnConvertEnum.getKeyByDesc(tmp.getTurnTypeName()));
                    }

                    List<CrossLaneDataHistPoExtend> tmpList = tmpListMap.get(tmp.getDirType());
                    if (tmpList == null) {
                        tmpList = new ArrayList<>();
                    }
                    tmpList.add(tmp);
                    tmpListMap.put(tmp.getDirType(), tmpList);
                }
            }
            for (Map.Entry<Integer, List<CrossLaneDataHistPoExtend>> tmpEntry : tmpListMap.entrySet()) {
                Map<String, Object> mapList = new HashMap<>();
                String dirDesc = BaseEnum.SignalDirectionEnum.getNameByCode(tmpEntry.getKey());
                mapList.put("scopeCode", null);
                mapList.put("isLost", true);
                mapList.put("dirName", dirDesc + "进口");
                mapList.put("scopeName", crossName);
                mapList.put("dirType", tmpEntry.getKey());
                mapList.put("list", tmpEntry.getValue());
                allList.add(mapList);
            }
        }
        //=========================================================================================================================================================//
        allList = allList.stream().sorted(Comparator.comparing(o -> Integer.valueOf(o.get("dirType").toString()))).collect(Collectors.toList());

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("scopeList", scopeList);

        return jsonObject;
    }

    @Override
    public JSONObject findCrossExitRoadLaneInfo(String crossId, String startTime, String endTime, String groupType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis(groupType, startTime, endTime);
        //======================================================================================================//
        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();

        List<CrossLaneDataHistPoExtend> list = greenwaveHistoryMapper.findCrossExitRoadLaneIndex(params);

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
        //按时间分组分组
        Map<String, List<CrossLaneDataHistPoExtend>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getCrossId() + "_" + o.getLaneNo() + "_" + o.getDirType() + "_" + o.getTurnType(), TreeMap::new, Collectors.toList()));

        Set<String> scopeList = new TreeSet<>();

        for (Map.Entry<String, List<CrossLaneDataHistPoExtend>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();
            String[] sps = key.split("_");
            String laneNo = sps[1];
            Integer dirType = Integer.parseInt(sps[2]);
            String dirName = null;
            String turnDesc = null;
            String dirTurnDesc = null;

            Map<String, Object> mapList = new HashMap<>();

            mapList.put("laneNo", laneNo);
            dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
            turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[3]));
            dirTurnDesc = dirName + "出口" + turnDesc + laneNo + "车道";
            scopeList.add(dirTurnDesc);

            //补充缺少时段数据，保留时段字段默认值
            List<CrossLaneDataHistPoExtend> value = this.processData(entry, groupType, sortedSet, startTime);

            mapList.put("scopeCode", key);
            mapList.put("scopeName", dirTurnDesc);
            mapList.put("list", value);
            allList.add(mapList);

        }


        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("scopeList", scopeList);

        return jsonObject;
    }

    @Override
    public JSONObject findCrossExitRoadDirInfo(String crossId, String startTime, String endTime, String groupType, Integer objectType) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossId);
        params.put("startDate", startTime);
        params.put("endDate", endTime);
        params.put("groupType", groupType);

        //存放时段
        Set<String> sortedSet = EsDateIndexUtil.getTimeGranularityAxis(groupType, startTime, endTime);

        //======================================================================================================//
        //存放所有数据
        List<Map<String, Object>> allList = new ArrayList<>();

        List<CrossLaneDataHistPoExtend> list = new ArrayList<>();
        if (objectType == 1) {
            list = greenwaveHistoryMapper.findCrossExitRoadIndex(params);
        } else if (objectType == 2) {
            list = greenwaveHistoryMapper.findCrossExitRoadLaneIndex(params);
        }

        list = list.stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());
        //按时间分组分组
        Map<String, List<CrossLaneDataHistPoExtend>> groupByDir = list.stream().collect(Collectors.groupingBy(o -> o.getCrossId() + "_" + o.getLaneNo() + "_" + o.getDirType() + "_" + o.getTurnType(), TreeMap::new, Collectors.toList()));

        Set<String> scopeList = new TreeSet<>();

        for (Map.Entry<String, List<CrossLaneDataHistPoExtend>> entry : groupByDir.entrySet()) {
            String key = entry.getKey();
            String[] sps = key.split("_");
            String laneNo = sps[1];
            Integer dirType = Integer.parseInt(sps[2]);
            String dirName = null;
            String turnDesc = null;
            String dirTurnDesc = null;

            Map<String, Object> mapList = new HashMap<>();

            mapList.put("laneNo", laneNo);
            if (objectType == 1) {
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                dirTurnDesc = dirName + "出口";
            } else if (objectType == 2) {
                dirName = BaseEnum.SignalDirectionEnum.getNameByCode(dirType);
                turnDesc = TurnConvertEnum.getDescByKey(Integer.valueOf(sps[3]));
                dirTurnDesc = dirName + "出口" + turnDesc + laneNo + "车道";
            }
            scopeList.add(dirTurnDesc);

            //补充缺少时段数据，保留时段字段默认值
            List<CrossLaneDataHistPoExtend> value = this.processData(entry, groupType, sortedSet, startTime);


            mapList.put("scopeCode", key);
            mapList.put("scopeName", dirTurnDesc);
            mapList.put("list", value);
            allList.add(mapList);
        }

        JSONObject jsonObject = new JSONObject();
        jsonObject.put("timeList", sortedSet);
        jsonObject.put("dataList", allList);
        jsonObject.put("scopeList", scopeList);

        return jsonObject;
    }

    /**
     * 补充缺少时段数据，保留时段字段默认值
     *
     * @param entry
     * @param groupType
     * @param sortedSet
     * @param startTime
     */
    private List<CrossLaneDataHistPoExtend> processData(Map.Entry<String, List<CrossLaneDataHistPoExtend>> entry, String groupType, Collection<String> sortedSet, String startTime) {
        //按方向排序
        List<CrossLaneDataHistPoExtend> value = entry.getValue().stream().sorted(Comparator.comparing(o -> o.getStartTime())).collect(Collectors.toList());

        //有实际数据的时刻点
        Set<String> timeList = new HashSet<>();
        if (Objects.equals("4", groupType)) {
            //天粒度
            timeList.addAll(value.stream().map(po -> DateUtil.formatDate(po.getStartTime(), EsDateIndexUtil.YMD_FORMATTER)).collect(Collectors.toSet()));
        } else if (Objects.equals("3", groupType)) {
            //小时粒度
            timeList.addAll(value.stream().map(po -> DateUtil.formatDate(po.getStartTime(), "yyyy-MM-dd HH:00:00")).collect(Collectors.toSet()));
        } else {
            //小时分钟粒度
            timeList.addAll(value.stream().map(po -> DateUtil.formatDate(po.getStartTime(), "yyyy-MM-dd HH:mm:00")).collect(Collectors.toSet()));
        }
        //补充缺少时段数据，保留时段字段默认值
        for (String timeSec : sortedSet) {
            if (!timeList.contains(timeSec)) {
                CrossLaneDataHistPoExtend tmp = new CrossLaneDataHistPoExtend();
                tmp.setLaneNo(value.get(0).getLaneNo());
                tmp.setTurnType(value.get(0).getTurnType());
                tmp.setDirType(value.get(0).getDirType());
                tmp.setTimeAxis(timeSec);
               // String parseTime = startTime.substring(0, startTime.lastIndexOf(" ") + 1) + timeSec;
                if (Objects.equals("4", groupType)) {
                    tmp.setStartTime(DateTime.parse(timeSec, DateTimeFormat.forPattern(EsDateIndexUtil.YMD_FORMATTER)).toDate());
                } else {
                    tmp.setStartTime(DateTime.parse(timeSec, DateTimeFormat.forPattern("yyyy-MM-dd HH:mm:ss")).toDate());
                }
                value.add(tmp);
            }
        }
        value = value.stream().sorted(Comparator.comparing(CrossLaneDataHistPoExtend::getStartTime)).collect(Collectors.toList());

        //==========================================================================================================//
        //时间轴
        value.forEach(o -> {
            if (Objects.equals("4", groupType)) {
                o.setTimeAxis(DateUtil.formatDate(o.getStartTime(), EsDateIndexUtil.YMD_FORMATTER));
            } else if (Objects.equals("3", groupType)) {
                o.setTimeAxis(DateUtil.formatDate(o.getStartTime(), EsDateIndexUtil.H_FORMATTER));
            } else {
                o.setTimeAxis(DateUtil.formatDate(o.getStartTime(), EsDateIndexUtil.HM_FORMATTER));
            }
        });

        return value;
    }

    private List<String> getTimeAxisList(String groupType,List<String> timeList) {
        //存放时段
        List<String> formatTimeList = new ArrayList<>();
        if (Objects.equals("4", groupType)) {
            //天粒度
            formatTimeList.addAll(timeList);
        } else {
            //小时分钟粒度
            formatTimeList.addAll(timeList.stream().map(dt -> dt.substring(dt.indexOf(" "),dt.lastIndexOf(":"))).collect(Collectors.toList()));
        }
        return  formatTimeList;
    }
}
